/*
 **
 ** Copyright 2019 Karlsruhe Institute of Technology, Chair of Dependable Nano Computing
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 ** and associated documentation files (the "Software"), to deal in the Software without restriction, 
 ** including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 ** and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
 ** so, subject to the following conditions:
 **
 ** The above copyright notice and this permission notice shall be included in all copies or substantial 
 ** portions of the Software.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
 ** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 ** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 ** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 ** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 **
 */

#include "dfathread.h"
#include <chrono>

DFAThread::DFAThread(QObject *parent, const std::shared_ptr<FPGA> &dfafpga) : QThread(parent), dfafpga(dfafpga) {
    *(this->encEna) = false;
    *(this->faultEna) = false;
    *(this->quitThread) = false;
}

void DFAThread::fillRandom(Block &target) {
    for (unsigned int i = 0; i < 16; i++) {
        target[i] = this->dist(this->gen);
    }
}

void DFAThread::run() {
    while(!*(this->quitThread)) {
        if (*(this->encEna) && *(this->faultEna)) {
            Block plaintext;
            Block cipher;
            Block faultcipher;
            this->fillRandom(plaintext);
            if(this->dfafpga->encrypt(plaintext, cipher) && this->dfafpga->encryptFaulty(plaintext, faultcipher)) {
                if (cipher != faultcipher) {
                    emit faultInjected(cipher, faultcipher);
                }
                emit encryptionFinished(plaintext, cipher);
                emit encryptionFinished(plaintext, faultcipher);
            }
        } else if (*(this->encEna)) {
            Block plaintext;
            Block cipher;
            this->fillRandom(plaintext);
            if (this->dfafpga->encrypt(plaintext, cipher)) {
                emit encryptionFinished(plaintext, cipher);
            }
        }
    }
    this->quit();
}



void DFAThread::toggleEnc() {
    *(this->encEna) = !*(this->encEna);
}

void DFAThread::setEnc(bool status) {
    *(this->encEna) = status;
}

bool DFAThread::getEnc() const {
    return *(this->encEna);
}

void DFAThread::toggleFault() {
    *(this->faultEna) = !*(this->faultEna);
}

void DFAThread::setFault(bool status) {
    *(this->faultEna) = status;
}

bool DFAThread::getFault() const {
    return *(this->faultEna);
}

void DFAThread::setQuit() {
    *(this->quitThread) = true;
}
