/*
 **
 ** Copyright 2019 Karlsruhe Institute of Technology, Chair of Dependable Nano Computing
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 ** and associated documentation files (the "Software"), to deal in the Software without restriction, 
 ** including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 ** and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
 ** so, subject to the following conditions:
 **
 ** The above copyright notice and this permission notice shall be included in all copies or substantial 
 ** portions of the Software.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
 ** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 ** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 ** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 ** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 **
 */

#ifndef DFATHREAD_H
#define DFATHREAD_H

#include <QThread>
#include <cinttypes>
#include <array>
#include <memory>
#include <random>
#include "fpga.h"
#include "types.h"

class DFAThread : public QThread
{
    Q_OBJECT
private:
    const std::shared_ptr<bool> encEna = std::shared_ptr<bool>(new bool);
    const std::shared_ptr<bool> faultEna = std::shared_ptr<bool>(new bool);
    const std::shared_ptr<bool> quitThread = std::shared_ptr<bool>(new bool);
    const std::shared_ptr<FPGA> dfafpga;
    std::default_random_engine gen;
    std::uniform_int_distribution<uint8_t> dist = std::uniform_int_distribution<uint8_t>(0, 255);
    void fillRandom(Block &target);
public:
    DFAThread(QObject *parent = nullptr, const std::shared_ptr<FPGA> &dfafpga = nullptr);
    void run() override;
    void toggleEnc();
    void setEnc(bool status);
    bool getEnc() const;
    void toggleFault();
    void setFault(bool status);
    bool getFault() const;
    void setQuit();
signals:
    void encryptionFinished(const Block &plaintext, const Block &cipher);
    void faultInjected(const Block &correct_cipher, const Block &faulty_cipher);
};

#endif // DFATHREAD_H
