/*
 **
 ** Copyright 2019 Karlsruhe Institute of Technology, Chair of Dependable Nano Computing
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 ** and associated documentation files (the "Software"), to deal in the Software without restriction, 
 ** including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 ** and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
 ** so, subject to the following conditions:
 **
 ** The above copyright notice and this permission notice shall be included in all copies or substantial 
 ** portions of the Software.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
 ** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 ** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 ** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 ** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 **
 */

#ifndef FPGA_H
#define FPGA_H

#include <string>
#include <cinttypes>
#include <vector>
#include <array>
#include <termios.h>
#include <exception>
#include "types.h"

class FPGA
{
private:
    static const unsigned int UART_TIMEOUT = 3000;
    unsigned int ro_activecycles = 1;
    unsigned int ro_cycles = 2;
    unsigned int sca_tracelen = 280;
    int serial_fd;
    speed_t convRate(unsigned int baudrate) const;
    bool encryptGeneric(const Block &plaintext, Block &cipher, uint8_t ro_ena);
public:
    FPGA(const std::string &serialport, unsigned int baudrate);
    ~FPGA();
    void setROParameters(unsigned int ro_activecycles, unsigned int ro_cycles);
    void setTracelen(unsigned int sca_tracelen);
    void setCycles(unsigned int ro_cycles);
    void setActivecycles(unsigned int ro_activecycles);
    void crash();
    void reset();
    void resetBlock();
    void release();
    bool inReset() const;
    unsigned int getCycles(void) const;
    unsigned int getActivecycles(void) const;
    bool encrypt(const Block &plaintext, Block &cipher);
    bool encryptSCA(const Block &plaintext, Block &cipher, Sensordata &sensordata);
    bool encryptFaulty(const Block &plaintext, Block &cipher);
};

#endif // FPGA_H
