/*
 **
 ** Copyright 2019 Karlsruhe Institute of Technology, Chair of Dependable Nano Computing
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 ** and associated documentation files (the "Software"), to deal in the Software without restriction, 
 ** including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 ** and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
 ** so, subject to the following conditions:
 **
 ** The above copyright notice and this permission notice shall be included in all copies or substantial 
 ** portions of the Software.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
 ** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 ** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 ** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 ** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 **
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <memory>
#include <array>
#include <string>
#include <list>
#include "fpga.h"
#include "AESCracker.h"
#include "types.h"
#include "dfathread.h"
#include "scathread.h"
#include "cpa.h"

namespace Ui {
class MainWindow;
}

Q_DECLARE_METATYPE(Block);
Q_DECLARE_METATYPE(Sensordata);
Q_DECLARE_METATYPE(Correlation);
Q_DECLARE_METATYPE(CorrelationProgress);

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

private slots:
    void on_toggleDFAUART_clicked();

    void on_roFreq_editingFinished();

    void on_roDutyCycle_editingFinished();

    void encryptionFinishedDFA(const Block &plaintext, const Block &cipher);

    void encryptionFinishedSCA(const Block &plaintext, const Block &cipher, const Sensordata &sensordata);

    void faultInjected(const Block &correct_cipher, const Block &faulty_cipher);

    void correlationUpdated(const Correlation &corr, const CorrelationProgress &prog);

    void on_toggleEnc_clicked();

    void on_toggleFault_clicked();

    void on_toggleShowEnc_clicked();

    void on_resetRecovery_clicked();

    void on_dfafpgaReset_clicked();

    void on_toggleSCAUART_clicked();

    void on_scafpgaReset_clicked();

    void on_scaToggleEnc_clicked();

    void on_resetCPA_clicked();

    void on_ppoint_editingFinished();

    void on_pstep_editingFinished();

    void on_byteSel_valueChanged(int arg1);

    void on_bitSel_valueChanged(int arg1);

    void on_modeSel_currentIndexChanged(int index);

    void on_toggleCrashUART_clicked();

    void on_crashfpgaReset_clicked();

    void on_startCrash_clicked();

    void on_dosReprogram_clicked();

private:
    static const unsigned int FPGA_FREQ = 60000000;
    static const Block key;
    static const Block roundkey;
    std::array<unsigned int, 4> num_faults_per_cat = {{0, 0, 0, 0}};
    Ui::MainWindow *ui;
    std::shared_ptr<DFAThread> dfathread;
    std::shared_ptr<SCAThread> scathread;
    std::shared_ptr<FPGA> dfaFPGA;
    std::shared_ptr<FPGA> scaFPGA;
    std::shared_ptr<FPGA> crashFPGA;
    AESCracker cracker;
    std::array<std::array<std::array<std::list<double>, 256>, 8>, 16> progress;
    QString arrayToString(const Block &a) const;
    QString faultyArrayToString(const Block &correct, const Block &faulty) const;
    std::string byteToString(uint8_t a) const;
    bool showEnc = false;
};

#endif // MAINWINDOW_H
