/*
 **
 ** Copyright 2019 Karlsruhe Institute of Technology, Chair of Dependable Nano Computing
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 ** and associated documentation files (the "Software"), to deal in the Software without restriction, 
 ** including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 ** and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
 ** so, subject to the following conditions:
 **
 ** The above copyright notice and this permission notice shall be included in all copies or substantial 
 ** portions of the Software.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
 ** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 ** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 ** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 ** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 **
 */

#ifndef SCATHREAD_H
#define SCATHREAD_H

#include <QThread>
#include <memory>
#include <random>
#include "fpga.h"
#include "types.h"
#include "cpa.h"

class SCAThread : public QThread
{
    Q_OBJECT
public:
    SCAThread(QObject *parent = nullptr, const std::shared_ptr<FPGA> &scafpga = nullptr);
    static const unsigned int TRACELEN = 280;
    void run() override;
    void toggleEnc();
    void setEnc(bool status);
    bool getEnc() const;
    void setQuit();
    void setCurrentBit(unsigned int bit);
    void setCurrentByte(unsigned int byte);
    void setProgressStep(unsigned int pstep);
    void setProgressPoint(int ppoint);
    void setCurrentMode(CPAMode m);
    void resetCPAs();
    unsigned int getNumTraces() const;
    unsigned int getCurrentByte() const;
    unsigned int getProgressStep() const;
    void getCurrentCorrelation(Correlation &corr) const;
    CPAMode getCurrentMode() const;
private:
    static const unsigned int PROGRESS_HIST = 100;
    const std::shared_ptr<FPGA> scafpga;
    const std::shared_ptr<bool> encEna = std::shared_ptr<bool>(new bool);
    const std::shared_ptr<bool> reset = std::shared_ptr<bool>(new bool);
    const std::shared_ptr<bool> quitThread = std::shared_ptr<bool>(new bool);
    const std::shared_ptr<unsigned int> current_byte = std::shared_ptr<unsigned int>(new unsigned int);
    const std::shared_ptr<unsigned int> current_bit = std::shared_ptr<unsigned int>(new unsigned int);
    const std::shared_ptr<unsigned int> pstep = std::shared_ptr<unsigned int>(new unsigned int);
    const std::shared_ptr<int> ppoint = std::shared_ptr<int>(new int);
    std::array<std::array<std::shared_ptr<CPA<TRACELEN>>, 8>, 16> cpas;
    std::array<std::array<CorrelationProgress, 8>, 16> progress;
    const std::shared_ptr<CPAMode> current_mode = std::shared_ptr<CPAMode>(new CPAMode);
    std::default_random_engine gen;
    std::uniform_int_distribution<uint8_t> dist = std::uniform_int_distribution<uint8_t>(0, 255);
    void fillRandom(Block &target);
signals:
    void encryptionFinished(const Block &plaintext, const Block &ciphertext, const Sensordata &sensordata);
    void correlationUpdated(const Correlation &corr, const CorrelationProgress &prog);
};

#endif // SCATHREAD_H
