----------------------------------------------------------------------------------
-- COPYRIGHT (c) 2015 ALL RIGHT RESERVED
--
-- KRYPTOGRAPHIE AUF PROGRAMMIERBARER HARDWARE: BASYS3 BOARD FRAMEWORK
----------------------------------------------------------------------------------



-- IMPORTS
----------------------------------------------------------------------------------
LIBRARY IEEE;
library ice;
use ice.vcomponent_vital.all;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;



-- ENTITY
----------------------------------------------------------------------------------
ENTITY MainControler IS
    PORT ( CLK              : IN    STD_LOGIC;
           CLK_SLOW         : IN    STD_LOGIC;
           RST				: IN	STD_LOGIC;
           -- UART CONTROL -----------------------------------
           UART_RESET       : OUT   STD_LOGIC;
           TX_ENABLE        : OUT   STD_LOGIC;
           TX_IDLE          : IN    STD_LOGIC;
           DATA_VALID       : IN    STD_LOGIC;
           switch_send_ciphertext : OUT STD_LOGIC;
           -- CIPHER CONTROL ---------------------------------
           CIPHER_RESET     : OUT   STD_LOGIC;
           CIPHER_DONE      : IN    STD_LOGIC;
           -- REGISTER CONTROL -------------------------------
           PT_CONTROL       : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
           CT_CONTROL       : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
           K_CONTROL        : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
           -- BRAM CONTROL -----------------------------------
           BR_EN            : OUT   STD_LOGIC;
           BR_WEA           : OUT   STD_LOGIC;
           ADR_COUNT        : OUT   STD_LOGIC_VECTOR(8 DOWNTO 0));
END MainControler;



-- ARCHITECTURE: FSM
----------------------------------------------------------------------------------
ARCHITECTURE FSM OF MainControler IS

    -- STATES --------------------------------------------------------------------
    TYPE STATES IS (S_RESET, S_IDLE, S_RECEIVE_PLAINTEXT, S_INIT, S_WAIT1, S_WAIT2, S_ENCRYPT, S_WAIT3, S_SEND_CIPHERTEXT, S_SEND_SENSORDATA);
    SIGNAL CURRENT_STATE, NEXT_STATE : STATES := S_RESET;
    
    -- SIGNALS -------------------------------------------------------------------
    SIGNAL CNT_RST, CNT_EN  : STD_LOGIC;
    SIGNAL COUNT            : UNSIGNED(9 DOWNTO 0);
	SIGNAL UART_VALID		: STD_LOGIC_VECTOR(1 DOWNTO 0);
	SIGNAL UART_IDLE		: STD_LOGIC_VECTOR(1 DOWNTO 0);
    SIGNAL ADR_CNT_RST, ADR_CNT_EN : STD_LOGIC;
    SIGNAL ADR_COUNT_INTERNAL : UNSIGNED(8 DOWNTO 0);

BEGIN
    ADR_COUNT <= std_logic_vector(ADR_COUNT_INTERNAL);
    
    Buffering : PROCESS(CLK)
    BEGIN
    	IF RISING_EDGE(CLK) THEN
    		IF (RST = '0') THEN
    			UART_VALID <= (OTHERS => '0');
    			UART_IDLE <= (OTHERS => '0');
			ELSE
				UART_VALID <= UART_VALID(0) & DATA_VALID;
				UART_IDLE <= UART_IDLE(0) & TX_IDLE;
			END IF;
    	END IF;
    END PROCESS;

    -- ADRESS PROCESS -----------------------------------------------------------
    ADR_Counter : PROCESS(CLK)
    BEGIN
        IF RISING_EDGE(CLK) THEN
            IF (ADR_CNT_RST = '1') THEN
                ADR_COUNT_INTERNAL <= (OTHERS => '0');
            ELSIF (ADR_CNT_EN = '1') THEN
                ADR_COUNT_INTERNAL <= ADR_COUNT_INTERNAL + 1;
            END IF;
        END IF;
    END PROCESS;
    ------------------------------------------------------------------------------

    -- COUNTER PROCESS -----------------------------------------------------------
    Counter : PROCESS(CLK)
    BEGIN
        IF RISING_EDGE(CLK) THEN
            IF (CNT_RST = '1') THEN
                COUNT <= (OTHERS => '0');
            ELSIF (CNT_EN = '1') THEN
                COUNT <= COUNT + 1;
            END IF;
        END IF;
    END PROCESS;
    ------------------------------------------------------------------------------
    
	-- STATE REGISTER PROCESS ----------------------------------------------------
	StateRegister : PROCESS(CLK)
	BEGIN
		IF RISING_EDGE(CLK) THEN
			IF (RST = '0') THEN
				CURRENT_STATE <= S_RESET;
			ELSE
				CURRENT_STATE <= NEXT_STATE;
			END IF;
		END IF;
	END PROCESS;
	------------------------------------------------------------------------------

        -- OUTPUT SIGNALS AND STATE TRANSITION PROCESS -------------------------------
        OutputAndTransition : PROCESS(CURRENT_STATE, DATA_VALID, COUNT, CIPHER_DONE, TX_IDLE, CLK_SLOW)
        BEGIN
            -- DEFAULT ASSIGNMENTS  --------------------------------------------------
            PT_CONTROL      <= "000";
            CT_CONTROL      <= "000";
            K_CONTROL       <= "000";
            
            UART_RESET      <= '0';
            TX_ENABLE       <= '0';     
            switch_send_ciphertext <= '1';
            CIPHER_RESET    <= '1';
            
            CNT_RST         <= '0';
            CNT_EN          <= '0'; 
            
            ADR_CNT_RST     <= '0';
            ADR_CNT_EN      <= '0';
            BR_EN           <= '0';
            BR_WEA          <= '0';                  
            NEXT_STATE      <= CURRENT_STATE;
            
            
            --------------------------------------------------------------------------
            
            -- CASE EVALUATION -------------------------------------------------------
            CASE CURRENT_STATE IS
                ----------------------------------------------------------------------
                WHEN S_RESET                =>  UART_RESET          <= '1';
                								PT_CONTROL(0)		<= '1';
                								CT_CONTROL(0)		<= '1';
                                                NEXT_STATE          <= S_IDLE;
                ----------------------------------------------------------------------
                WHEN S_IDLE                 =>  CNT_RST             <= '1';
                                                
                                                NEXT_STATE          <= S_RECEIVE_PLAINTEXT;
                ----------------------------------------------------------------------
                WHEN S_RECEIVE_PLAINTEXT    =>  IF (UART_VALID = "01") THEN
                                                    CNT_EN          <= '1'; 
                                                    PT_CONTROL(1)   <= '1';
                                                END IF;
                                                
                                                IF (COUNT = "10000") THEN
                                                    CNT_RST         <= '1';
                                                    NEXT_STATE      <= S_INIT;
                                                END IF;
                ----------------------------------------------------------------------
                WHEN S_INIT                 =>  IF (CLK_SLOW = '0') THEN
                                                    NEXT_STATE      <= S_WAIT1;
                                                END IF;
                ----------------------------------------------------------------------
                WHEN S_WAIT1                =>  IF (CLK_SLOW = '1') THEN
                                                    ADR_CNT_RST     <= '1';
                                                    CNT_RST         <= '1';
                                                    NEXT_STATE      <= S_WAIT2;
                                                END IF;
                ----------------------------------------------------------------------
                WHEN S_WAIT2                =>  BR_EN               <= '1'; 
                                                ADR_CNT_EN          <= '1';
                                                BR_WEA              <= '1'; -- record?
                                                
                                                CNT_EN              <= '1';
                
                                                IF (COUNT = "100000") THEN
                                                    CNT_RST         <= '1';
                                                    NEXT_STATE      <= S_ENCRYPT;
                                                END IF;
                                                                                        
                ----------------------------------------------------------------------
                WHEN S_ENCRYPT              =>  CIPHER_RESET        <= '0';
                
                                                BR_EN               <= '1'; 
                                                ADR_CNT_EN          <= '1';
                                                BR_WEA              <= '1'; -- record?
                                                 
                                                IF (CIPHER_DONE = '1') THEN
                                                    CT_CONTROL(1)   <= '1';
                                                    CT_CONTROL(2)   <= '1';
                                                    NEXT_STATE      <= S_WAIT3;
                                                END IF;
                ----------------------------------------------------------------------
                WHEN S_WAIT3                =>  BR_EN               <= '1'; 
                                                ADR_CNT_EN          <= '1';
                                                BR_WEA              <= '1'; -- record?
                                                
                                                CNT_EN              <= '1';
                
                                                IF (COUNT = "100000") THEN
                                                    CNT_RST         <= '1';
                                                    NEXT_STATE      <= S_SEND_CIPHERTEXT;
                                                END IF;
                ----------------------------------------------------------------------
                WHEN S_SEND_CIPHERTEXT      =>  TX_ENABLE           <= '1';
                                                IF (UART_IDLE = "01") THEN
                                                    CNT_EN          <= '1'; 
                                                    CT_CONTROL(1)   <= '1';
                                                END IF;
                                                
                                                IF (COUNT = "10000") THEN
                                                    CNT_RST         <= '1';
                                                    ADR_CNT_RST     <= '1';
                                                    NEXT_STATE      <= S_SEND_SENSORDATA;
                                                END IF;  
                                                
                WHEN S_SEND_SENSORDATA      =>  TX_ENABLE           <= '1';
                                                switch_send_ciphertext <= '0';
                                                
                                                 IF (UART_IDLE = "01") THEN
                                                       BR_EN               <= '1'; 
                                                       ADR_CNT_EN    <= '1';    
                                                       CNT_EN        <= '1';   
                                                       --CT_CONTROL(1)   <= '1';
                                                   END IF;
                                                
                                               IF (COUNT = "100011000") THEN --280
                                                   CNT_RST         <= '1';
                                                   ADR_CNT_RST     <= '1';
                                                   NEXT_STATE      <= S_IDLE;  
                                                END IF;  
                                                
                                                                                              
            END CASE;        
            --------------------------------------------------------------------------
            
        END PROCESS;
        ------------------------------------------------------------------------------
END FSM;
