----------------------------------------------------------------------------------
-- COPYRIGHT (c) 2015 ALL RIGHT RESERVED
--
-- KRYPTOGRAPHIE AUF PROGRAMMIERBARER HARDWARE: iCE40HX8K BOARD FRAMEWORK
----------------------------------------------------------------------------------



-- IMPORTS
----------------------------------------------------------------------------------
LIBRARY IEEE;
library ice;
use ice.vcomponent_vital.all;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;



-- ENTITY
----------------------------------------------------------------------------------
ENTITY LatticeiCE40HX8KFramework is
    GENERIC (
           SENSOR_BITS : Integer := 63;
           SENSOR_CODED : Integer := 6; -- log2(SENSOR_BITS)
           
           SENSOR_INITIAL_DELAY : Integer :=  10
           
           -- 100mhz
           -- SENSOR_INITIAL_DELAY : Integer := 16 -- was 16 for board 43, board 56 corevcc@3.290V
                                                -- 14 for board 49
           );
    PORT ( CLK      : IN    STD_LOGIC;
           -- 4 BUTTONS ----------------------------------
           RSTIN   : IN    STD_LOGIC;
           -- 8 LED -------------------------------------
           LED      : OUT   STD_LOGIC_VECTOR( 7 DOWNTO 0);
           -- UART I / O ---------------------------------
           UART_RX  : IN    STD_LOGIC;
           UART_TX  : OUT   STD_LOGIC);
END LatticeiCE40HX8KFramework;



-- ARCHITECTURE: STRUCTURAL
----------------------------------------------------------------------------------
ARCHITECTURE Structural OF LatticeiCE40HX8KFramework IS

    -- COMPONENTS ----------------------------------------------------------------
    COMPONENT SB_RAM512x8
      PORT (
        wclk : IN STD_LOGIC;
        rclk : IN STD_LOGIC;
        wclke : IN STD_LOGIC;
        rclke : IN STD_LOGIC;
        re : IN STD_LOGIC;
        we : IN STD_LOGIC;
        waddr : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
        raddr : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
        rdata : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
        wdata : IN STD_LOGIC_VECTOR(7 DOWNTO 0)
      );
    END COMPONENT;
    
    component debounce
    port (
    	clkin : in std_logic;
    	asynchin : in std_logic;
    	synchout : out std_logic
    );
    end component;
    
    COMPONENT RegisterSerPar is
        GENERIC ( SIZE_SER : POSITIVE := 8; SIZE_PAR : POSITIVE := 128);
        PORT ( CLK    : IN  STD_LOGIC;
               RST    : IN  STD_LOGIC;
               EN     : IN  STD_LOGIC;
               SEL    : IN  STD_LOGIC;
               DS     : IN  STD_LOGIC_VECTOR((SIZE_SER - 1) DOWNTO 0);
               DP     : IN  STD_LOGIC_VECTOR((SIZE_PAR - 1) DOWNTO 0);
               QS     : OUT STD_LOGIC_VECTOR((SIZE_SER - 1) DOWNTO 0);
               QP     : OUT STD_LOGIC_VECTOR((SIZE_PAR - 1) DOWNTO 0));
    END COMPONENT;
    
    COMPONENT uart IS
    	generic ( fpga_frequency : integer := 96000000;
	  				bit_rate : integer := 115200);
        PORT ( CLK          : IN    STD_LOGIC;
               RESET        : IN    STD_LOGIC;
               -- TX SIGNALS ---------------------------------
               DATA_IN      : IN    STD_LOGIC_VECTOR(7 DOWNTO 0);
               TX_ENABLE    : IN    STD_LOGIC;
               TX_IDLE      : OUT   STD_LOGIC;
               TX_OUT       : OUT   STD_LOGIC;
               -- RX SIGNALS ---------------------------------
               DATA_OUT     : OUT   STD_LOGIC_VECTOR(7 DOWNTO 0);
               DATA_VALID   : OUT   STD_LOGIC;
               RX_IN        : IN    STD_LOGIC);
    END COMPONENT;
    
    COMPONENT MainControler IS
        PORT ( CLK              : IN    STD_LOGIC;
               CLK_SLOW         : IN    STD_LOGIC;
               RST				: IN 	STD_LOGIC;
               -- UART CONTROL -----------------------------------
               UART_RESET       : OUT   STD_LOGIC;
               TX_ENABLE        : OUT   STD_LOGIC;
               TX_IDLE          : IN    STD_LOGIC;
               DATA_VALID       : IN    STD_LOGIC;
               switch_send_ciphertext : OUT   STD_LOGIC;
               -- CIPHER CONTROL ---------------------------------
               CIPHER_RESET     : OUT   STD_LOGIC;
               CIPHER_DONE      : IN    STD_LOGIC;
               -- REGISTER CONTROL -------------------------------
               PT_CONTROL       : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
               CT_CONTROL       : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
               K_CONTROL        : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
               -- BRAM CONTROL -----------------------------------
               BR_EN            : OUT   STD_LOGIC;
               BR_WEA           : OUT   STD_LOGIC;
               ADR_COUNT        : OUT   STD_LOGIC_VECTOR(8 DOWNTO 0));
    END COMPONENT;
    
    component freq_div is
        Port ( clk     : in  STD_LOGIC;
               clk_out : out  std_logic);
    end component;    
    
    component aes_tiny is
        Port ( clk : in  STD_LOGIC;
               rst : in  STD_LOGIC;
               din : in  STD_LOGIC_VECTOR (127 downto 0);
               key : in  STD_LOGIC_VECTOR (127 downto 0);
               dout : out  STD_LOGIC_VECTOR (127 downto 0);
               done : out  STD_LOGIC);
    end component;
    
    component lattice_pll
        Port ( PLLOUTGLOBALA : out STD_LOGIC;
               PLLOUTGLOBALB : out STD_LOGIC;
               PLLOUTCOREA : out STD_LOGIC;
               PLLOUTCOREB : out STD_LOGIC;
               REFERENCECLK : in STD_LOGIC;
               RESET : in STD_LOGIC);
    end component;
    
    component lattice_pll2
        Port ( PLLOUTGLOBALA : out STD_LOGIC;
               PLLOUTGLOBALB : out STD_LOGIC;
               PLLOUTCOREA : out STD_LOGIC;
               PLLOUTCOREB : out STD_LOGIC;
               REFERENCECLK : in STD_LOGIC;
               RESET : in STD_LOGIC);
    end component;
    
    component latticesense
        generic (
            initlen  : integer;
            linelen : integer
        );
        port (
          clkin		   : in  std_ulogic;
          valout 	   : out std_logic_vector(linelen-1 downto 0)
        );
    end component;
    
    component clkcheck
    	generic (
    		freqhz : integer);
    	port (
    		clkin : in std_logic;
    		rstin : in std_logic;
    		ledout : out std_logic);
    end component;
    
    component tdc_decode
      generic (
         input_len : integer;
         output_width : integer
       );
      port (
         clk : in std_ulogic;
         rst : in std_ulogic;
         chainvalue_i : in std_logic_vector(input_len-1 downto 0);
         coded_o : out std_logic_vector(output_width-1 downto 0)
       );
    end component;
    
   -- SIGNALS -------------------------------------------------------------------
   SIGNAL PT_CONTROL, CT_CONTROL, K_CONTROL : STD_LOGIC_VECTOR(  2 DOWNTO 0);
   
   SIGNAL KEY, PLAINTEXT, CIPHERTEXT        : STD_LOGIC_VECTOR(127 DOWNTO 0);
   
   SIGNAL DATA_IN, DATA_OUT                 : STD_LOGIC_VECTOR(  7 DOWNTO 0);
   SIGNAL TX_ENABLE, TX_IDLE                : STD_LOGIC;
   SIGNAL DATA_VALID                        : STD_LOGIC;
   SIGNAL UART_RESET                        : STD_LOGIC;
   
   SIGNAL CIPHER_RESET, CIPHER_DONE  : STD_LOGIC;
   
   SIGNAL SlowCLK : STD_LOGIC;
   
   -------------------------
   
   signal clk24mhz : STD_LOGIC;
   signal clk96mhz : STD_LOGIC;
   signal clk12mhz : STD_LOGIC;
   signal clk48mhz :STD_LOGIC;
   
   signal delay_line_s : STD_LOGIC_VECTOR(SENSOR_BITS-1 downto 0);
   signal dec_tdc_s : STD_LOGIC_VECTOR(SENSOR_CODED-1 downto 0);
   
   signal dummy_rst : STD_LOGIC;
   
   signal re : STD_LOGIC;
   signal we :STD_LOGIC;
   signal rwaddr : STD_LOGIC_VECTOR(8 DOWNTO 0);
   
   signal sensor_data_to_send : STD_LOGIC_VECTOR(7 DOWNTO 0);
   signal sensor_data_to_store : STD_LOGIC_VECTOR(7 DOWNTO 0);
   
   signal data_for_uart : STD_LOGIC_VECTOR(7 DOWNTO 0);
   signal switch_send_ciphertext : STD_LOGIC;
   
   signal rst : std_logic;
   
BEGIN
    sensor_data_to_store(7 downto SENSOR_CODED) <= (others => '0');
    sensor_data_to_store(SENSOR_CODED-1 downto 0) <= dec_tdc_s;
    
    LED(5 downto 0) <= dec_tdc_s(5 downto 0);

    data_for_uart <= DATA_IN when (switch_send_ciphertext = '1') else  sensor_data_to_send;
    
    SlowCLK <= clk12mhz;

	debounce_rst : debounce
	port map (
		clkin => clk96mhz,
		asynchin => RSTIN,
		synchout => rst
	);
	
	clkcheck24_inst : clkcheck
	generic map (
		freqhz => 24000000
	)
	port map (
		clkin => clk24mhz,
		rstin => rst,
		ledout => LED(7)
	);
	
	clkcheck96_inst : clkcheck
	generic map (
		freqhz => 96000000
	)
	port map (
		clkin => clk96mhz,
		rstin => rst,
		ledout => LED(6)
	);

    -- UART MODULE ---------------------------------------------------------------
    UARTInstance : uart
    GENERIC MAP (
    	fpga_frequency => 12000000,
    	bit_rate => 500000
    )
    PORT MAP (
        clk         => clk12mhz,
        reset       => UART_RESET,
        data_in     => data_for_uart,
        tx_enable   => TX_ENABLE,
        tx_idle     => TX_IDLE,
        tx_out      => UART_TX,
        data_out    => DATA_OUT,
        data_valid  => DATA_VALID,
        rx_in       => UART_RX
    );
    ------------------------------------------------------------------------------
 

    -- SERIAL-PARALLEL REGISTER (PLAINTEXT, CIPHERTEXT, KEY) ---------------------
    Register_Plaintext : RegisterSerPar
    GENERIC MAP (SIZE_SER => 8, SIZE_PAR => 128)
    PORT MAP (
        CLK => clk48mhz,
        RST => PT_CONTROL(0),
        EN  => PT_CONTROL(1),
        SEL => PT_CONTROL(2),
        DS  => DATA_OUT,
        DP  => (OTHERS => '0'),
        QS  => OPEN,
        QP  => PLAINTEXT
    );
    
    KEY <= x"2b7e151628aed2a6abf7158809cf4f3c";
    
    Register_Ciphertext : RegisterSerPar
    GENERIC MAP (SIZE_SER => 8, SIZE_PAR => 128)
    PORT MAP (
        CLK => clk48mhz,
        RST => CT_CONTROL(0),
        EN  => CT_CONTROL(1),
        SEL => CT_CONTROL(2),
        DS  => (OTHERS => '0'),
        DP  => CIPHERTEXT,
        QS  => DATA_IN,
        QP  => OPEN
    );
    ------------------------------------------------------------------------------
    
    -- CIPHER INSTANCE -----------------------------------------------------------
    AESCore : aes_tiny
    PORT MAP (
        clk       => SlowCLK,
        rst       => CIPHER_RESET,
        done      => CIPHER_DONE,
        key       => KEY,
        din       => PLAINTEXT,
        dout      => CIPHERTEXT
    );
    
    ------------------------------------------------------------------------------
                
    -- MAIN CONTROL FINITE STATE MACHINE -----------------------------------------
    FSM : MainControler
    PORT MAP (
        CLK             => clk48mhz,
        CLK_SLOW        => SlowCLK,
        RST				=> rst,
        -- UART CONTROL -----------------------------------
        UART_RESET      => UART_RESET,
        TX_ENABLE       => TX_ENABLE,
        TX_IDLE         => TX_IDLE,
        DATA_VALID      => DATA_VALID,
        switch_send_ciphertext => switch_send_ciphertext,
        -- CIPHER CONTROL ---------------------------------
        CIPHER_RESET    => CIPHER_RESET,
        CIPHER_DONE     => CIPHER_DONE,
        -- REGISTER CONTROL -------------------------------
        PT_CONTROL      => PT_CONTROL,
        CT_CONTROL      => CT_CONTROL,
        K_CONTROL       => K_CONTROL,
        -- BRAM CONTROL -----------------------------------
        BR_EN           => re,
        BR_WEA          => we,
        ADR_COUNT       => rwaddr
                
    );
    ------------------------------------------------------------------------------
    
    clkgen : lattice_pll
         port map ( PLLOUTGLOBALA => clk24mhz,
                   PLLOUTGLOBALB => clk12mhz,
                   REFERENCECLK => CLK,
                   RESET => '1' );
    clkgen2 : lattice_pll2
         port map ( PLLOUTGLOBALA => clk96mhz,
                   PLLOUTGLOBALB => clk48mhz,
                   REFERENCECLK => CLK,
                   RESET => '1' );
    
    -- sensor instance
    tdc : latticesense
        generic map (
            initlen => SENSOR_INITIAL_DELAY, -- No. of the LUTs+Latches for pre-delay
            linelen => SENSOR_BITS -- No. of the final bits to be latched for reading
        )
        port map (
            clkin => clk48mhz,
            valout => delay_line_s
        );
    
    dummy_rst <= '0';
  
    -- decoder/encoder instance
    en_tdc : tdc_decode
        generic map(input_len => SENSOR_BITS, output_width => SENSOR_CODED)
        port map (clk => clk48mhz, rst => rst, chainvalue_i => delay_line_s, coded_o => dec_tdc_s);       
    
    mem : SB_RAM512x8
          PORT MAP (
            rclk => clk48mhz,
            wclk => clk48mhz,
            re => re,
            we => we,
            wclke => '1',
            rclke => '1',
            raddr => rwaddr,
            waddr => rwaddr,
            wdata => sensor_data_to_store,
            rdata => sensor_data_to_send
          );
    
END Structural;
