----------------------------------------------------------------------------------
--
-- Copyright 2019 Karlsruhe Institute of Technology, Chair of Dependable Nano Computing
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software
-- and associated documentation files (the "Software"), to deal in the Software without restriction, 
-- including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
-- and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
-- so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial 
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity tdc_decode is
    generic ( input_len : integer := 64; output_width : integer := 6 );
    port ( clk : in std_ulogic;
           rst : in std_ulogic;
		   chainvalue_i : in std_logic_vector(input_len-1 downto 0);
		   coded_o : out std_logic_vector(output_width-1 downto 0));
end tdc_decode;

architecture beh of tdc_decode is
  
  signal dec_reg : std_logic_vector(output_width-1 downto 0) := (others => '0');
  signal tdc_input_buf_reg : std_logic_vector(input_len-1 downto 0);

begin
  
  decodeLogic : process(tdc_input_buf_reg, clk, rst)
    variable one_sum : integer := 0;
    variable sel0 : std_logic_vector(1 downto 0);
  begin
    -- combinational
    one_sum := 0;
     
    for k in 0 to input_len-1 loop
        if tdc_input_buf_reg(k) = '1' then
            one_sum := one_sum+1;
        --else
				--exit;
		  end if;
    end loop;
    
    --if (one_msb > 1) then
     --   sel0 :=  tdc_input_buf_reg(one_msb-1) & tdc_input_buf_reg(one_msb-2);
      --  case sel0 is 
       --     when "10" => 
        --        one_msb := one_msb-1;
         --   when others => 
          --      one_msb := one_msb;
        --end case;
    --end if;

    -- sequential
    if rising_edge(clk) then
      if rst = '0' then
       dec_reg <= (others => '1');
       tdc_input_buf_reg <= (others => '1');
      else
       dec_reg <= std_logic_vector(to_unsigned(one_sum,output_width));
       tdc_input_buf_reg <= chainvalue_i;
      end if;
    end if;
    
  end process;
  
  coded_o <= dec_reg;

end beh;