----------------------------------------------------------------------------------
--
-- Copyright 2019 Ruhr University Bochum, Horst Görtz Institute for IT Security
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software
-- and associated documentation files (the "Software"), to deal in the Software without restriction, 
-- including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
-- and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
-- so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial 
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity DFF_SSR is
	port (
		CLK		: in std_logic;					-- input clock
		SR		: in std_logic;					-- set/reset (high)
		SRINIT	: in std_logic;					-- reset value
		CE		: in std_logic;					-- enable signal
		D		: in std_logic;					-- input bit
		Q		: out std_logic					-- output bit
	);
end DFF_SSR;

architecture Behavioral of DFF_SSR is
begin

	-- D-flipflop, synchronous reset, reset high
	FlipFlop : process(clk)
	begin
		if rising_edge(clk) then
			if SR = '1' then
				Q <= SRINIT;
			else
				if CE = '1' then
					Q <= D;
				end if;
			end if;					-- reset
		end if;						-- clk
	end process FlipFlop;

end Behavioral;

