----------------------------------------------------------------------------------
--
-- Copyright 2019 Ruhr University Bochum, Horst Görtz Institute for IT Security
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software
-- and associated documentation files (the "Software"), to deal in the Software without restriction, 
-- including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
-- and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
-- so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial 
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity nBitReg_SSR is
	generic (
		BIT_WIDTH	: integer range 1 to 256 := 16				-- register size in bits
	);
	port (
		CLK		: in std_logic;									-- input clock
		SR		: in std_logic;									-- set/reset (high)
		SRINIT	: in std_logic_vector(BIT_WIDTH-1 downto 0);	-- init value
		CE		: in std_logic;									-- enable signal
		DIN		: in std_logic_vector(BIT_WIDTH-1 downto 0);	-- input word
		DOUT	: out std_logic_vector(BIT_WIDTH-1 downto 0)	-- output word
	);
end nBitReg_SSR;

architecture Behavioral of nBitReg_SSR is

	-- --------------------------------------------- 
	--                Components
	-- --------------------------------------------- 
	
	-- synchronously reset D-FlipFlop
	component DFF_SSR is
		port (
			CLK		: in std_logic;					-- input clock
			SR		: in std_logic;					-- set/reset (high)
			SRINIT	: in std_logic;					-- reset value
			CE		: in std_logic;					-- enable signal
			D		: in std_logic;					-- input bit
			Q		: out std_logic					-- output bit
		);
	end component DFF_SSR;

begin

	-- Register, synchronous reset, reset high
	Reg : for i in 1 to BIT_WIDTH generate
		FF : DFF_SSR
			port map (
				CLK => CLK,
				SR	=> SR,
				SRINIT => SRINIT(i-1),
				CE	=> CE,
				D	=> DIN(i-1),
				Q	=> DOUT(i-1)
			);
	end generate;

end Behavioral;

