----------------------------------------------------------------------------------
--
-- Copyright 2019 Ruhr University Bochum, Horst Görtz Institute for IT Security
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software
-- and associated documentation files (the "Software"), to deal in the Software without restriction, 
-- including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
-- and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do 
-- so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or substantial 
-- portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--
----------------------------------------------------------------------------------

-- IMPORTS
----------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;



-- ENTITY
----------------------------------------------------------------------------------
ENTITY uart IS
    generic ( fpga_frequency : integer := 100000000;
	   bit_rate : integer := 128000);
    PORT ( clk                      : IN    STD_LOGIC;
           reset                    : IN    STD_LOGIC;
           -- TX SIGNALS ---------------------------------
           data_in                  : IN    STD_LOGIC_VECTOR(7 DOWNTO 0);
           tx_enable                : IN    STD_LOGIC;
           tx_idle                  : OUT   STD_LOGIC;
           tx_out                   : OUT   STD_LOGIC;
           -- RX SIGNALS ---------------------------------
           data_out                 : OUT   STD_LOGIC_VECTOR(7 DOWNTO 0);
           data_valid               : OUT   STD_LOGIC;
           rx_in                    : IN    STD_LOGIC);
END uart;



-- ARCHITECTURE: STRUCTURAL
----------------------------------------------------------------------------------
ARCHITECTURE Structural OF uart IS

    -- COMPONENTS ----------------------------------------------------------------
    COMPONENT pc_uart_receive IS
        generic ( fpga_frequency : integer := 100000000;
	       bit_rate : integer := 128000);
        PORT ( clk              : IN    STD_LOGIC;
               reset            : IN    STD_LOGIC;
               rs232_bit        : IN    STD_LOGIC;
               data_out         : OUT   STD_LOGIC_VECTOR(7 DOWNTO 0);
               data_valid       : OUT   STD_LOGIC);
    END COMPONENT;
    
    COMPONENT pc_uart_send IS
        generic ( fpga_frequency : integer := 100000000;
	       bit_rate : integer := 128000);
        PORT ( clk              : IN    STD_LOGIC;
               reset            : IN    STD_LOGIC;
               data_in          : IN    STD_LOGIC_VECTOR(7 DOWNTO 0);
               rs232_bit        : OUT   STD_LOGIC;
               send_enable      : IN    STD_LOGIC;
               ready_for_data   : OUT   STD_LOGIC);
    END COMPONENT;

BEGIN

    uart_receiver : pc_uart_receive
    generic map (
    	fpga_frequency => fpga_frequency,
    	bit_rate => bit_rate
    )
    PORT MAP (
        clk => clk,
        reset => reset,
        rs232_bit => rx_in,
        data_out => data_out,
        data_valid => data_valid
    );
     
    uart_sender : pc_uart_send
    generic map (
    	fpga_frequency => fpga_frequency,
    	bit_rate => bit_rate
    )
    PORT MAP (
        clk => clk,
        reset => reset,
        data_in => data_in,
        rs232_bit => tx_out,
        send_enable => tx_enable,
        ready_for_data => tx_idle
    );

END Structural;

